<?php

namespace Drupal\flickr_integration_suite;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Http\ClientFactory;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\key\KeyRepositoryInterface;

/**
 * Flickr API Service provider.
 */
class FlickrIntegrationSuiteApiProvider {

  use StringTranslationTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The key repository interface.
   *
   * @var \Drupal\key\KeyRepositoryInterface
   */
  protected $keyRepository;

  /**
   * The http_client_factory service.
   *
   * @var \Drupal\Core\Http\ClientFactory
   */
  protected $httpClientFactory;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The Flickr config.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $flickrConfig;

  /**
   * The Flickr Api Endpoint.
   */
  protected string $flickrApiEndpoint;

  /**
   * The Flickr Api Key.
   */
  protected string $flickrApiKey;

  /**
   * Constructs a new FlickrIntegrationSuiteApiProvider object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\key\KeyRepositoryInterface $key_repository
   *   The Key Repository interface.
   * @param \Drupal\Core\Http\ClientFactory $http_client_factory
   *   The http_client_factory service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(ConfigFactoryInterface $config_factory, KeyRepositoryInterface $key_repository, ClientFactory $http_client_factory, MessengerInterface $messenger) {
    $this->configFactory = $config_factory;
    $this->keyRepository = $key_repository;
    $this->httpClientFactory = $http_client_factory;
    $this->messenger = $messenger;

    $this->flickrConfig = $this->configFactory->get('flickr_integration_suite.settings');
    $this->flickrApiEndpoint = $this->flickrConfig->get('api_endpoint');
    $this->flickrApiKey = $this->keyRepository->getKey($this->flickrConfig->get('api_key'))->getKeyValue();
  }

  /**
   * The flickr.photosets.getPhotos service.
   *
   * Get the list of photos in a set.
   */
  public function photosetsGetPhotos($photoset_id, $per_page = 500, $page = 1, $extras = '') {
    if (!$this->flickrApiKey) {
      $this->messenger->addError($this->t('Flickr API credentials are not set. It can be set on the <a href=":config_page">configuration page</a>.',
        [':config_page' => Url::fromRoute('flickr_integration_suite.settings_form')]
      ));
      return FALSE;
    }

    $parameters = [
      'method' => 'flickr.photosets.getPhotos',
      'api_key' => $this->flickrApiKey,
      'photoset_id' => $photoset_id,
      'format' => 'json',
      'nojsoncallback' => 1,
      'per_page' => $per_page,
      'page' => $page,
      'extras' => $extras,
    ];

    // TO DO: cacheable.
    $client = $this->httpClientFactory->fromOptions();
    $response = $client->get($this->flickrApiEndpoint, ['query' => $parameters]);

    if (!$response) {
      $this->messenger->addError($this->t('An error has occurred with Flickr API. No response given.'));
    }
    else {
      $data = json_decode($response->getBody());

      if (isset($data->stat) && $data->stat == 'fail') {
        if (isset($data->message) && !empty($data->message)) {
          $this->messenger->addError($this->t('Flickr API Error - :message',
            [':message' => $data->message]
          ));
        }
      }

      return $data;
    }

    return FALSE;
  }

}
